'use strict';

const debug = require('debug')('process');
const db = require('../../database/mongo.js');

const Inbox = require('@intelisis/g4-runtime-api-inbox');
const Steps = require('@intelisis/g4-runtime-api-steps');
const Process = require('@intelisis/g4-process');
const BusinessApps = require('@intelisis/g4-runtime-api-business-apps');
const _ = require('underscore');

const AWSSDK = require('aws-sdk');
const AWS = new AWSSDK.S3(__config.aws);

exports.affect = function Affect(socket, io, detail) {
  // get inboxId and archive inbox
  const inboxId = detail.inboxId;
  const inbox = new Inbox({ db, tenantName: socket.get('tenant') });

  debug('affect', inboxId);

  // get details from client request
  const positionId = detail.positionId;
  const data = detail.data;

  // get position item
  let step = new Steps({ db, tenantName: socket.get('tenant'), _id: positionId });

  step.fetch().then(() => {
    const position = step.get('position');
    const processId = position.processId;

    step = position.activity;

    const formName = step.form.name;

    const proc = new Process({ _id: processId }, { db, tenant: socket.get('tenant'), socket, io, AWS });

    return proc.fetch({ silent: true }).then(() => {
      const payload = proc.get('payload');
      payload.Form[formName] = data;

      proc.set('payload', payload);
      proc.set('step', proc.get('elements').get(step._id), { silent: true });

      proc.next().then(() => {
        io.to(socket.get('_id')).emit('inbox:archive', inboxId);
        inbox.affectOne(inboxId);
      }).catch((err) => {
        socket.error(err);
      });
    });
  }).catch((err) => {
    const out = err;
    out.message += ` ${inboxId}`;
    debug(out);
    socket.error(out);
  });
};

const signalSchema = require('../../database/schemas/signal');

exports.affectSignal = function affectSignal(socket, io, packet) {
  const signalId = packet.signalId;
  const catalogName = packet.catalogName;
  const catalogItemId = packet.catalogItemId;

  if (!signalId || !catalogName || !catalogItemId) {
    return socket.error(new Error('signalId, catalogName or catalogItemId missing'));
  }

  const tenantName = socket.get('tenant');

  const tenant = db.useDb(tenantName);

  const model = tenant.model('signal', signalSchema);
  model.findById(signalId).then((signalResult) => {
    const processId = signalResult.processId;
    const proc = new Process({ _id: processId }, { socket, io, tenant: tenantName, AWS, db });
    return proc.fetch({ trigger: false }).then(() => {
      const payload = proc.get('payload');
      const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);

      kernel.get(`/catalogs/${tenantName}/${catalogItemId}`, (err, response, body) => {
        if (err) return socket.error(err);
        const unmappedData = body.response;

        const documents = proc.get('documents');
        const bappId = proc.get('bappId');

        const formObj = _.findWhere(documents, { _id: catalogItemId });
        if (!formObj) {
          return socket.error(new Error('Document Not Found'));
        }
        const formName = formObj.name;
        const bapp = new BusinessApps({ _id: bappId, tenantName, db });
        return bapp.fetch().then(() =>
          bapp.getFormByName(formName).then((form) => {
            const data = proc.reverseMapper(unmappedData, form);

            payload.Form[formName] = data;
            proc.set('payload', payload);
            const step = proc.get('elements').get(signalResult.extra._id);
            proc.set('step', step);

            proc.once('inbox:sent', () => {
              socket.emit('inbox:navigate');
            });

            return proc.next().catch((procErr) => {
              socket.error(procErr);
            });
          })
        );
      });
    });
  }).catch((err) => {
    debug(err);
    socket.error(err);
  });
};

exports.affectMovSignal = function affectMovSignal(socket, io, packet) {
  const signalId = packet.signalId;
  const documentName = packet.documentName;
  const documentId = packet.documentId;

  if (!signalId || !documentName || !documentId) {
    return socket.error(new Error('signalId, documentName or documentId missing'));
  }

  const tenantName = socket.get('tenant');

  const tenant = db.useDb(tenantName);

  const model = tenant.model('signal', signalSchema);

  model.findById(signalId).then((signalResult) => {
    const processId = signalResult.processId;
    const proc = new Process({ _id: processId }, { socket, io, tenant: tenantName, AWS, db });
    
    return proc.fetch({ trigger: false }).then(() => {
      const payload = proc.get('payload');
      const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);

      kernel.get(`documents/singlerecord?tenant_id=${tenantName}&document_id=${documentId}`, (err, response, body) => {
        if (err) return socket.error(err);
        const data = body.response;

        const documents = proc.get('documents');
        const bappId = proc.get('bappId');

        const formObj = _.findWhere(documents, { _id: documentId });

        if (!formObj) {
          return socket.error(new Error('Document Not Found'));
        }
        const formName = formObj.name;
        const bapp = new BusinessApps({ _id: bappId, tenantName, db });
        return bapp.fetch().then(() =>
          bapp.getFormByName(formName).then((form) => {
            // const data = proc.reverseMapper(unmappedData, form);

            payload.Form[formName] = data;
            proc.set('payload', payload);
            const step = proc.get('elements').get(signalResult.extra._id);
            proc.set('step', step);

            proc.once('inbox:sent', () => {
              socket.emit('inbox:navigate');
            });

            return proc.next().catch((procErr) => {
              socket.error(procErr);
            });
          })
        );
      });
    });
  }).catch((err) => {
    debug(err);
    socket.error(err);
  });
};

exports.affectProcessSignal = function affectMovSignal(socket, io, packet) {
  const processId = packet.processId;
  const documentName = packet.documentName;
  const documentId = packet.documentId;

  if (!processId || !documentName || !documentId) {
    return socket.error(new Error('processId, documentName or documentId missing'));
  }

  const tenantName = socket.get('tenant');

  const tenant = db.useDb(tenantName);

  const model = tenant.model('signal', signalSchema);

  const proc = new Process({ _id: processId }, { socket, io, tenant: tenantName, AWS, db });
  
  return proc.fetch({ trigger: false }).then(() => {
    const payload = proc.get('payload');
    const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);

    kernel.get(`documents/singlerecord?tenant_id=${tenantName}&document_id=${documentId}`, (err, response, body) => {
      if (err) return socket.error(err);
      const data = body.response;

      const documents = proc.get('documents');
      const bappId = proc.get('bappId');

      let formName;
      if(data){
        formName = data.formName;
      }
      else
        formName = {};

      if(!formName){
        formName = documentName;
      }

      const bapp = new BusinessApps({ _id: bappId, tenantName, db });
      return bapp.fetch().then(() =>
        bapp.getFormByName(formName).then((form) => {
          // const data = proc.reverseMapper(unmappedData, form);

          payload.Form[formName] = data;

          proc.set('payload', payload);

          // console.log('0_________', payload);

          // const step = proc.get('elements').get(signalResult.extra._id);

          // console.log('1', payload);

          // proc.set('step', step);

          proc.once('inbox:sent', () => {
            socket.emit('inbox:navigate');
          });

          return proc.next().catch((procErr) => {
            socket.error(procErr);
          });
        })
      );
    });
  });
};
