'use strict';

const debug = require('debug')('catalog');
const db = require('../../database/mongo.js');
const _ = require('underscore');
const BusinessApps = require('@intelisis/g4-runtime-api-business-apps');
const Process = require('@intelisis/g4-process');
const AWSSDK = require('aws-sdk');
const AWS = new AWSSDK.S3(__config.aws);

const ITEMS_PER_PAGE = 25;
const abstractions = require('./abstractions');
const createActions = require('../../utils').createActions;
const customCatActions = require('../../utils').customCatActions;
const getCatalogsPerPage = abstractions.getCatalogsPerPage;
const getCatalogs = abstractions.getCatalogs;

const Inbox = require('@intelisis/g4-runtime-api-inbox');
const Steps = require('@intelisis/g4-runtime-api-steps');

const customForm = { name: 'Custom', pages: [{ }], width: 100, height: 100 };
const exportActions = (catalogName) => {
  return {
    left: [{
      icon: 'actionbar-information',
      className: 'information',
      status: 'off',
    }, {
      icon: 'actionbar-print',
      className: 'print',
      status: 'off',
    }, {
      icon: 'actionbar-cancel',
      className: 'cancel',
      status: 'on',
      callback: `(function() {
        inbox.request('archive');
      })()`,
    }, {
      status: 'on',
      icon: 'actionbar-attachments',
      callback: `(function () {
        socket.emit('superSecretMaintenance:export', { catalogName: '${catalogName}' });
        const catalogChannel = Backbone.Radio.channel('catalogs');
        catalogChannel.trigger('modal:close');
      })()`,
    }],
  };
};

exports.getCatalogListHTTP = (req, res, next) => {
  const tenantName = req.tenant;

  const kernel = require('request-json').createClient(`http://${req.user.tenant.kernel_ip}/`);
  kernel.get(`/subtypes?tenant_id=${tenantName}&type=entity`, (err, response, body) => {
    if (err) return next(err);
    res.send(body.response);
  });
};

exports.getCatalogListWS = (socket) => {
  const options = {
    kernelIp: socket.get('kernel_ip'),
    tenantName: socket.get('tenant'),
  };
  getCatalogs(options)
  .then((result) => {
    socket.emit('catalogs:list', result);
  })
  .catch((err) => {
    socket.error(err);
  });
};

exports.getCatalogItemsPerPageHTTP = (req, res, next) => {
  const tenantName = req.tenant;
  const fields = req.query.fields;
  const page = req.query.page;
  const url = `/subtypes/records?tenant_id=${tenantName}&fields=${fields}&subType=${req.query.catalog}&page=${page}&limit=${ITEMS_PER_PAGE}`;
  const kernel = require('request-json').createClient(`http://${req.user.tenant.kernel_ip}/`);
  kernel.get(url, (err, response, body) => {
    if (err) return next(err);
    res.send(body.response);
  });
};

exports.getCatalogItemsPerPageWS = (socket, io, packet) => {
  const options = {
    tenantName: socket.get('tenant'),
    kernelIp: socket.get('kernel_ip'),
    fields: '_id,code%20as%20Clave,name%20as%20Nombre,description%20as%20Descripcion,kind%20as%20Tipo,status%20as%20Estatus,class%20as%20Clase',
    catalog: packet.catalog,
    page: packet.page,
  };
  getCatalogsPerPage(options)
  .then((result) => {
    socket.emit('catalogs:items', result);
  }).catch((err) => {
    socket.error(err);
  });
};

const launchNewProcess = (tenantName, tenant, catalogName, socket, io, unmappedData, catalogItemId) => {
  const bapp = new BusinessApps({ db, tenantName });

  bapp.businessApps.find({ 'forms.name': catalogName }).select('diagrams forms').lean().then((bapps) => {
    let foundDiagram;
    let finalBapp;

    for (let bapp of bapps) {
      const diagrams = bapp.diagrams;
      for (let diagram of diagrams) {
        const pools = diagram.pools;
        for (let pool of pools) {
          const droppableZones = pool.droppableZones;
          for (let zone of droppableZones) {
            const elements = zone.elementsCollection;
            for (let element of elements) {
              if (element.subtype === 'user' && element.form && element.form.name === catalogName) {
                foundDiagram = diagram;
                finalBapp = bapp;
              }
            }
          }
        }
      }
    }

    if (!foundDiagram) return socket.error(new Error('Diagram Not Found'));
    const enterStepId = foundDiagram.stepOmmit;
    if (!enterStepId) return socket.error(new Error('Default Step Not Found'));

    const bappId = finalBapp._id;

    const form = _.findWhere(finalBapp.forms, { name: catalogName });
    if (!form) return socket.error(new Error('Form Not Found'));

    const proc = new Process(null, { tenant, db, socket, AWS, io });
    const data = proc.reverseMapper(unmappedData, form);

    proc.set('diagram', foundDiagram);
    const step = proc.get('elements').get(enterStepId);

    proc.set('step', step);
    proc.set('bappId', bappId);
    proc.save().then(() => {
      proc.set('documents', [{ _id: catalogItemId, name: catalogName }]);
      return proc.next().then(() =>
        proc.signal.find({ processId: proc.get('_id') }).then((results) => {
          const options = {
            entityId: catalogItemId,
            formName: form.name,
            tenantName,
            entityName: unmappedData.name,
          };
          customCatActions(socket, options, (error, presetActions) => {
            if (error) {
              socket.error(err);
            }
            const actions = createActions(results, [presetActions]);
            socket.emit('catalogs:show', { form, data, actions });
          });
        })
      );
    }).catch((procErr) => {
      socket.error(procErr);
    });
  });
};

exports.getCatalogItem = (socket, io, packet) => {
  debug('request item', packet);
  const catalogItemId = packet._id;
  const catalogName = packet.catalogName;
  const tenant = socket.get('tenant');
  const tenantName = tenant;
  if (!catalogItemId || !catalogName) {
    const error = new Error('catalogItemId or catalogName missing');
    debug(error);
    return socket.error(new Error('catalogItemId or catalogName missing'));
  }
  const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);

  kernel.get(`/catalogs/${tenantName}/${catalogItemId}`, (err, response, body) => {
    if (err) return socket.error(err);
    const unmappedData = body.response;

    /**
     * [if description]
     * @param  {[type]} !unmappedData [description]
     * @return {[type]}               [description]
     */
    if (!unmappedData) {
      socket.emit('catalogs:show', { form: customForm, actions: exportActions(catalogName) });
      return false;
    }

    /**
     * [processId description]
     * @type {[type]}
     */
    const processId = unmappedData.process_id;

    // if no process id
    if (!processId) {
      launchNewProcess(tenantName, tenant, catalogName, socket, io, unmappedData, catalogItemId);
    } else {
      const proc = new Process({ _id: processId }, { tenant, db, socket, AWS, io });

      proc.fetch({ trigger: false }).then(() => {
        const documents = proc.get('documents');
        const bappId = proc.get('bappId');
        const formObj = _.findWhere(documents, { _id: catalogItemId });

        if (!formObj) {
          return socket.error(new Error('Document Not Found'));
        }
        const formName = formObj.name;
        const bapp = new BusinessApps({ _id: bappId, tenantName: tenant, db });
        return bapp.fetch().then(() =>
          bapp.getFormByName(formName).then((form) => {
            const data = proc.reverseMapper(unmappedData, form);

            return proc.signal.find({ processId }).then((results) => {
              const options = {
                entityId: catalogItemId,
                formName,
                tenantName,
                entityName: unmappedData.name,
                processId,
              };

              customCatActions(socket, options, (error, presetActions) => {
                if (error) {
                  socket.error(err);
                }
                const actions = createActions(results, [presetActions]);
                socket.emit('catalogs:show', { form, data, actions });
              });
            });
          })
        );
      }).catch((procErr) => {
        if (procErr.message.indexOf('Process was deleted') > -1) {
          launchNewProcess(tenantName, tenant, catalogName, socket, io, unmappedData, catalogItemId);
        } else if (procErr.message.indexOf('business app not found') > -1) {
          launchNewProcess(tenantName, tenant, catalogName, socket, io, unmappedData, catalogItemId);
        } else {
          socket.error(procErr);
        }
      });
    }
  });
};

exports.fetchItemsByQuery = (req, res, next) => {
  const query = req.query;
  let key = query.key || '';

  if (key) {
    key = encodeURIComponent(key);
  }
  let catalogName = query.catalogName;
  if (!catalogName) {
    catalogName = query.__catalog;
  }
  if (!catalogName) {
    return next(new Error('Catalog name missing'));
  }
  const tenantName = req.tenant;
  const bapp = new BusinessApps({ db, tenantName });
  bapp.businessApps.findOne({ 'forms.name': catalogName }, { 'forms.$': 1 }).then((forms) => {
    if (!forms) {
      return next(new Error(`Form ${catalogName} not found in any of the BusinessApp installed.`));
    }
    const form = forms.forms[0];
    if (req.query.subType) catalogName = req.query.subType;
    let url = `/subtypes/search?tenant_id=${req.tenant}&subType=${catalogName}&key=${key}`;
    for (let queryKey in req.query) {
      if (queryKey !== 'catalogName' && queryKey !== 'subType' && queryKey !== 'key') {
        url += `&${queryKey}=${req.query[queryKey]}`;
      }
    }
    const kernel = require('request-json').createClient(`http://${req.user.tenant.kernel_ip}/`);
    const proc = new Process({ }, { tenant: tenantName, db, socket: { get() { return 'asd'; } }, AWS, io: {} });
    kernel.get(url, (err, response, body) => {
      if (err) return next(err);
      if (body.status === 200) {
        const items = body.response.response;
        const newItems = [];
        const labels = body.response.label;
        for (let item of items) {
          let key = item.name
          if (typeof labels !== 'undefined') {
            key = ''
            for (let label of labels.split(',')) {
              key = key + item[label] + ' ';
            }
          }
          const data = proc.reverseMapper(item, form);
          newItems.push({ _id: item._id, key: key, data });
        }
        res.send(newItems);
      } else {
        next(new Error('Kernel Error'));
      }
    });
  });
};

exports.deleteCatalogWS = function deleteCatalogWS(socket, io, packet) {
  const catalogId = packet._id;
  const tenant = socket.get('tenant');
  const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);
  return kernel.delete(`/catalogs/${tenant}/${catalogId}`, (err, response, body) => {
    if (err) return socket.error(err);
    if (body.status !== 200){
      return socket.error(new Error(body.description));
    }
    else{
      const tenantName = socket.get('tenant');
      const inbox = new Inbox({ db, tenantName });

      inbox.getActive({}).then((docs) => {
        for(let doc of docs){
          let step = new Steps({ db, tenantName: tenantName, _id: doc.position });

          step.fetch().then(() => {
            const position = step.get('position');
            const proc = new Process({ _id: position.processId }, { tenant, db, socket, AWS, io });

            proc.fetch({ trigger: false }).then(() => {
              if(proc.attributes.documents){
                for(let document of proc.attributes.documents){
                  if(document._id === packet._id){
                    const socketId = socket.get('_id');

                    io.to(socketId).emit('inbox:archive', doc._id);

                    inbox.archiveOne(doc._id).catch((err) => {
                      return socket.error(err);
                    });                    
                  }
                }
              }

            }).catch((procErr) => {
              return socket.error(procErr);
            });
          });
        }

      }).catch((err) => {
        return socket.error(err);
      });

      socket.emit('catalogs:delete', body.response);
    }
  });
};
