'use strict';

const debug = require('debug')('catalog');
const db = require('../../database/mongo.js');
const Movements = require('@intelisis/g4-runtime-api-catalog');
const Forms = require('@intelisis/g4-runtime-api-forms');
// const Process = require('@intelisis/g4-runtime-api-process');
const Process = require('@intelisis/g4-process');
const BusinessApps = require('@intelisis/g4-runtime-api-business-apps');
const ITEMS_PER_PAGE = 25;
const abstractions = require('./abstractions');
const createMovActions = require('../../utils').createMovActions;
const customMovActions = require('../../utils').customMovActions;
const request = require('request');
const getMovementsPerPage = abstractions.getMovementsPerPage;
const getMovements = abstractions.getMovements;
const _ = require('underscore');
const AWSSDK = require('aws-sdk');
const AWS = new AWSSDK.S3(__config.aws);
const Inbox = require('@intelisis/g4-runtime-api-inbox');
const Steps = require('@intelisis/g4-runtime-api-steps');

const customForm = { name: 'Custom', pages: [{ }], width: 100, height: 100 };

const exportActions = (catalogName) => {
  return {
    left: [{
      icon: 'actionbar-information',
      className: 'information',
      status: 'on'
    }, {
      icon: 'actionbar-print',
      className: 'print',
      status: 'off',
    }, {
      icon: 'actionbar-cancel',
      className: 'cancel',
      status: 'on',
      callback: `(function() {
        inbox.request('archive');
      })()`,
    }, {
      status: 'on',
      icon: 'actionbar-attachments',
      callback: `(function () {
        socket.emit('superSecretMaintenance:export', { catalogName: '${catalogName}' });
        const catalogChannel = Backbone.Radio.channel('movements');
        catalogChannel.trigger('modal:close');
      })()`,
    }],
  };
};

// exports.getMovementListHTTP = (req, res, next) => {
//   getMovements({ __catalog: '_', db, tenantName: req.tenant })
//   .then((result) => {
//     res.send(result);
//   })
//   .catch((err) => {
//     next(err);
//   });
// };

exports.getMovementListHTTP = (req, res, next) => {
  const tenantName = req.tenant;

  const kernel = require('request-json').createClient(`http://${req.user.tenant.kernel_ip}/`);
  kernel.get(`/documents?tenant_id=${tenantName}&type=entity`, (err, response, body) => {
    if (err) return next(err);
    res.send(body.response);
  });
};


exports.getMovementListWS = (socket) => {
  getMovements({ __catalog: '_', db, tenantName: socket.get('tenant') })
  .then((result) => {
    socket.emit('movements:list', result);
  })
  .catch((err) => {
    socket.error(err);
  });
};

// exports.getMovementsItemsPerPageHTTP = (req, res, next) => {
//   const movementName = req.query.catalog;
//   const page = req.query.page;
//   const fields = req.query.fields;
//   const tenantName = req.tenant;
//   getMovementsPerPage({ movementName, db, page, fields, tenantName, ITEMS_PER_PAGE })
//   .then((result) => {
//     res.send(result);
//   }).catch((err) => {
//     next(err);
//   });
// };

exports.getMovementsItemsPerPageHTTP = (req, res, next) => {
  const tenantName = req.tenant;
  const fields = req.query.fields;
  const page = req.query.page;
  const url = `/documents/records?tenant_id=${tenantName}&fields=${fields}&documentName=${req.query.catalog}&page=${page}&limit=${ITEMS_PER_PAGE}`;
  const kernel = require('request-json').createClient(`http://${req.user.tenant.kernel_ip}/`);
  kernel.get(url, (err, response, body) => {
    if (err) return next(err);
    res.send(body.response);
  });
};


exports.getMovementItemsPerPageWS = (socket, io, packet) => {
  const movementName = packet.catalog;
  const page = packet.page;
  const fields = packet.fields;
  const tenantName = socket.get('tenant');
  getMovementsPerPage({ movementName, db, page, fields, tenantName, ITEMS_PER_PAGE })
  .then((result) => {
    socket.emit('movements:items', result);
  }).catch((err) => {
    socket.error(err);
  });
};

const launchNewProcess = (tenantName, tenant, documentName, socket, io, unmappedData, documentId) => {
  const bapp = new BusinessApps({ db, tenantName });

  bapp.businessApps.find({ 'forms.name': documentName }).select('diagrams forms').lean().then((bapps) => {
    let foundDiagram;
    let finalBapp;

    for (let bapp of bapps) {
      const diagrams = bapp.diagrams;
      for (let diagram of diagrams) {
        const pools = diagram.pools;
        for (let pool of pools) {
          const droppableZones = pool.droppableZones;
          for (let zone of droppableZones) {
            const elements = zone.elementsCollection;
            for (let element of elements) {
              if (element.subtype === 'user' && element.form && element.form.name === documentName) {
                foundDiagram = diagram;
                finalBapp = bapp;
              }
            }
          }
        }
      }
    }

    if (!foundDiagram) return socket.error(new Error('Diagram Not Found'));
    const enterStepId = foundDiagram.stepOmmit;
    // if (!enterStepId) return socket.error(new Error('Default Step Not Found'));

    const bappId = finalBapp._id;

    const form = _.findWhere(finalBapp.forms, { name: documentName });
    if (!form) return socket.error(new Error('Form Not Found'));

    const proc = new Process(null, { tenant, db, socket, AWS, io });

    const data = proc.reverseMovMapper(unmappedData, form, socket.get('kernel_ip'), tenant, proc);

    proc.set('diagram', foundDiagram);
    const step = proc.get('elements').get(enterStepId);

    proc.set('step', step);
    proc.set('bappId', bappId);
    proc.save().then(() => {
      proc.set('documents', [{ _id: documentId, name: documentName }]);
      return proc.next().then(() =>
        proc.signal.find({ processId: proc.get('_id') }).then((results) => {
          const options = {
            documentId: documentId,
            formName: form.name,
            tenantName,
            entityName: unmappedData.name,
            stepLabel: '',
          };

          if(data.DatosGenerales.Estatus)
            options.status = data.DatosGenerales.Estatus;
          else
            options.status = data.Folios.Estatus;

          customMovActions(socket, options, (error, presetActions) => {
            if (error) {
              socket.error(err);
            }
            const actions = createMovActions(options.status, documentName, documentId, results, undefined, [presetActions]);  

            socket.emit('movements:show', { form, data, actions });
          });
        })
      );
    }).catch((procErr) => {
      socket.error(procErr);
    });
  });
};

const mergeModalFormState = (form, payload, formName) => {
  const formState = payload.State[formName];
  // check if there is metadata to extend
  if (formState) {
    // const section = formState.section;
    const pages = form.pages;
    
    for (let pageIndex in pages) {
      const page = pages[pageIndex];
      const sections = page.sections;
      for (let sectionIndex in sections) {
        const section = sections[sectionIndex];
        if (section.name === formState.section) {
          section.elements = formState.elements.map((column) => {
            const returnObj = {
              name: column.name,
              data: {
                label: column.label,
              },
              type: 'input',
              width: Number(column.width) || 140,
              height: Number(column.height) || 100,
              x: Number(column.x) || 0,
              y: Number(column.y) || 0,
              fromKernel: true,
              group: null,
              style: {
                extra: {
                  readonly: column.readOnly,
                },
              },
              helper: {
                options: {
                  required: false,
                },
                type: 'string',
              },
            };

            if (column.dataType === 'Boolean') {
              returnObj.makeGroup = true;
            }

            return returnObj;
          });

          let groupColumn = null;
          // check if we need to invent a fake group for the checkbox
          section.elements = _.reject(section.elements, (column) => {
            if (column.makeGroup) {
              groupColumn = column;
            }
            return column.makeGroup;
          });

          if (groupColumn) {
            const group = {
              type: 'group',
              name: groupColumn.name,
              group: null,
              fromKernel: true,
              helper: {
                options: {
                  optionType: 'checkbox',
                },
              },
              data: {
                label: groupColumn.label,
                value: false,
              },
              width: Number(groupColumn.width) || 140,
              height: Number(groupColumn.height) || 100,
              x: Number(groupColumn.x) || 0,
              y: Number(groupColumn.y) || 0,
            };
            section.elements.push(group);

            const checkbox = {
              type: 'checkbox',
              name: 'checkbox1',
              fromKernel: true,
              group: groupColumn.name,
              helper: {
                options: {},
              },
              data: {
                label: '',
                value: 'true',
              },
              width: Number(groupColumn.width) || 140,
              height: Number(groupColumn.height) || 100,
              x: Number(groupColumn.x) || 0,
              y: Number(groupColumn.y) || 0,
            };

            section.elements.push(checkbox);
          }
        }
      }
    }
  }
};

exports.getMovementItem = (socket, io, packet) => {

  debug('request item', packet);
  const documentId = packet._id;
  const documentName = packet.documentName;
  const tenant = socket.get('tenant');
  const tenantName = tenant;
  if (!documentId || !documentName) {
    const error = new Error('documentId or documentName missing');
    debug(error);
    return socket.error(new Error('documentId or documentName missing'));
  }
  const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);

  kernel.get(`/documents/singlerecord?tenant_id=${tenantName}&document_id=${documentId}`, (err, response, body) => {
    if (err) return socket.error(err);
    const unmappedData = body.response;

    let formName;
    if(unmappedData){
      formName = unmappedData.formName;
    }
    else
      formName = {};

    if(!formName){
      formName = documentName;
    }

    if (!unmappedData) {
      socket.emit('movements:show', { form: customForm, actions: exportActions(formName) });
      return false;
    }

    const processId = unmappedData.document.process_id;

    // if no process id
    if (!processId) {
      launchNewProcess(tenantName, tenant, formName, socket, io, unmappedData, documentId);
    } else {
      const proc = new Process({ _id: processId }, { tenant, db, socket, AWS, io });

      proc.fetch({ trigger: false }).then(() => {

        const bapp = new BusinessApps({ db, tenantName });

        bapp.businessApps.find({ 'forms.name': formName }).select('diagrams forms').lean().then((bapps) => {
          let foundDiagram;
          let finalBapp;

          for (let bapp of bapps) {
            const diagrams = bapp.diagrams;
            for (let diagram of diagrams) {
              const pools = diagram.pools;
              for (let pool of pools) {
                const droppableZones = pool.droppableZones;
                for (let zone of droppableZones) {
                  const elements = zone.elementsCollection;
                  for (let element of elements) {
                    if (element.subtype === 'user' && element.form && element.form.name === formName) {
                      foundDiagram = diagram;
                      finalBapp = bapp;
                    }
                  }
                }
              }
            }
          }

          if (!foundDiagram) return socket.error(new Error('Diagram Not Found'));

          const bappId = finalBapp._id;

          const form = _.findWhere(finalBapp.forms, { name: formName });
          if (!form) return socket.error(new Error('Form Not Found'));

          const data = proc.reverseMovMapper(unmappedData, form, socket.get('kernel_ip'), tenant, proc);

          mergeModalFormState(form, proc.attributes.payload, formName);

          proc.set('diagram', foundDiagram);

          proc.signal.find({ processId: processId }).then((results) => {
          
            const step = proc.attributes.step;

            const options = {
              documentId: documentId,
              formName: form.name,
              tenantName,
              entityName: unmappedData.name,
              stepLabel: step.attributes.label,
            };

            if(data.DatosGenerales.Estatus)
              options.status = data.DatosGenerales.Estatus;
            else
              options.status = data.Folios.Estatus;

            customMovActions(socket, options, (error, presetActions) => {
              if (error) {
                socket.error(err);
              }
              const actions = createMovActions(options.status, formName, documentId, results, {processId: processId, label: step.attributes.label}, [presetActions]);

              socket.emit('movements:show', { form, data, actions });
            });
          })
        });
      }).catch((procErr) => {
        if (procErr.message.indexOf('Process was deleted') > -1) {
          launchNewProcess(tenantName, tenant, formName, socket, io, unmappedData, documentId);
        } else if (procErr.message.indexOf('business app not found') > -1) {
          launchNewProcess(tenantName, tenant, formName, socket, io, unmappedData, documentId);
        } else {
          socket.error(procErr);
        }
      });
    }
  });
};

exports.cancelMovementWS = function cancelMovementWS(socket, io, packet) {
  const options = {
    uri: `http://${socket.get('kernel_ip')}/affectations`,
    method: 'POST',
    body: {
      document: {
        document_id: packet._id,
        tenant_id: socket.get('tenant'),
      },
      extra: {
        action: 'documentCancellation',
        partial: packet.partial
      },
    },
    json: true,
  };

  request(options, (err, body, response) => {    
    if (err) return socket.error(err);

    if (response.status !== 200) 
      return socket.error(new Error(response.description));
    else{
      const tenantName = socket.get('tenant');
      const inbox = new Inbox({ db, tenantName });
      
      inbox.getActive({}).then((docs) => {        
        for(let doc of docs){
          let step = new Steps({ db, tenantName: tenantName, _id: doc.position });

          step.fetch().then(() => {
            const position = step.get('position');
            const payload = position.payload;
            step = position.activity;

            if(payload.State.document_id === packet._id){              
              const socketId = socket.get('_id');
              io.to(socketId).emit('inbox:archive', doc._id);
              inbox.archiveOne(doc._id).catch((err) => {
                socket.error(err);
              });
            }
          });
        }

      }).catch((err) => {
        socket.error(err);
      });

      return socket.emit('movements:cancel', response);
    }
  });
};
