'use strict';
const createActions = require('../../utils').createActions;
const customActions = require('../../utils').customActions;
const debug = require('debug')('process');
const Forms = require('@intelisis/g4-runtime-api-forms');
const Steps = require('@intelisis/g4-runtime-api-steps');
const db = require('../../database/mongo.js');
const _ = require('underscore');
const dataSchema = require('../../database/schemas/data');
const abstractions = require('./abstractions');
const getAccess = abstractions.getAccess;
const saveAccess = abstractions.saveAccess;
const BusinessApps = require('@intelisis/g4-runtime-api-business-apps');

const set = (object, path, value) => {
  const a = path.split('.');
  let o = object;
  for (let i = 0; i < a.length - 1; i++) {
    const n = a[i];
    if (n in o) {
      o = o[n];
    } else {
      o[n] = {};
      o = o[n];
    }
  }
  o[a[a.length - 1]] = value;
};

const reverseMapper = (map, form) => {
  const mapped = { };
  const query = form.query;
  const template = query.template;
  const CatExtra = map.CatExtra;

  for (let page of form.pages) {
    for (let sectionCopy of page.sections) {
      const section = JSON.parse(JSON.stringify(sectionCopy));
      section.elements = _.reject(section.elements, (item) => {
        return item.type === 'label' || item.type === 'line';
      });
      // if not an array section
      if (section.helper && section.helper.options && section.helper.options.array) {

        if(CatExtra){
          for (let row of CatExtra) {
            const sectionName = row.properties.__sectionName;
            if (!mapped[sectionName]) {
              mapped[sectionName] = [];
            }

            const newRow = {};
            for (let element of section.elements) {
              const elementName = element.name;
              const elementTarget = elementName;

              let value = row.properties[elementTarget];

              if (value) {
                if(!map.texts){
                  map.texts = {}
                }

                if (map.texts[value]) {
                  value = {
                    id: value,
                    _id: value,
                    text: map.texts[value],
                  };
                }
                newRow[elementTarget] = value;
              }
            }
            if (Object.keys(newRow).length > 0) {
              newRow.rowId = row.rowID;
              mapped[sectionName].push(newRow);
            }
          }
        }
      } else {
        for (let element of section.elements) {
          const sectionName = section.name;
          const elementName = element.name;
          let elementTarget = elementName;
          let fromProperties = true;
          for (let name in template) {
            const value = template[name];
            if (value === `${sectionName}.${elementName}`) {
              fromProperties = false;
              elementTarget = name;
            }
          }

          let value;

          if (fromProperties) {
            if(map.properties)
              value = map.properties[elementTarget];
            else
              value = undefined;

          } else {
            value = map[elementTarget];
          }

          value = value || null;

          if (map.texts && map.texts[value]) {
            value = {
              id: value,
              text: map.texts[value],
              _id: value,
            };
          }

          set(mapped, `${sectionName}.${elementName}`, value);
        }
      }
    }
  }

  return mapped;
};

// TODO: only use the !position.position thing
//
const mergeFormState = (form, payload, formName) => {
  const formState = payload.State[formName];
  // check if there is metadata to extend
  if (formState) {
    // const section = formState.section;
    const pages = form.get('pages');
    for (let pageIndex in pages) {
      const page = pages[pageIndex];
      const sections = page.sections;
      for (let sectionIndex in sections) {
        const section = sections[sectionIndex];
        if (section.name === formState.section) {
          section.elements = formState.elements.map((column) => {
            const returnObj = {
              name: column.name,
              data: {
                label: column.label,
              },
              type: 'input',
              width: Number(column.width) || 140,
              height: Number(column.height) || 100,
              x: Number(column.x) || 0,
              y: Number(column.y) || 0,
              fromKernel: true,
              group: null,
              style: {
                extra: {
                  readonly: column.readOnly,
                },
              },
              helper: {
                options: {
                  required: false,
                },
                type: 'string',
              },
            };

            if (column.dataType === 'Boolean') {
              returnObj.makeGroup = true;
            }

            return returnObj;
          });

          let groupColumn = null;
          // check if we need to invent a fake group for the checkbox
          section.elements = _.reject(section.elements, (column) => {
            if (column.makeGroup) {
              groupColumn = column;
            }
            return column.makeGroup;
          });

          if (groupColumn) {
            const group = {
              type: 'group',
              name: groupColumn.name,
              group: null,
              fromKernel: true,
              helper: {
                options: {
                  optionType: 'checkbox',
                },
              },
              data: {
                label: groupColumn.label,
                value: false,
              },
              width: Number(groupColumn.width) || 140,
              height: Number(groupColumn.height) || 100,
              x: Number(groupColumn.x) || 0,
              y: Number(groupColumn.y) || 0,
            };
            section.elements.push(group);

            const checkbox = {
              type: 'checkbox',
              name: 'checkbox1',
              fromKernel: true,
              group: groupColumn.name,
              helper: {
                options: {},
              },
              data: {
                label: '',
                value: 'true',
              },
              width: Number(groupColumn.width) || 140,
              height: Number(groupColumn.height) || 100,
              x: Number(groupColumn.x) || 0,
              y: Number(groupColumn.y) || 0,
            };

            section.elements.push(checkbox);
          }
        }
      }
    }
  }
};

const mergeFormData = (originalData, payload, formName) => {
  const state = payload.State[formName];
  if (state) {
    const data = state.data;
    const section = state.section;
    const sectionData = [];
    if (data) {
      const columns = data.header;
      for (let row of data.data) {
        const newRow = {};
        for (let colIndex in row) {
          newRow[columns[colIndex]] = row[colIndex];
        }
        sectionData.push(newRow);
      }
      originalData[section] = sectionData;
    }
  }
};

const mergeFromTemps = (form, payload) => {
  const Temps = payload.State.Temps;
  function deepObjectExtend (target, source) {
      for (var prop in source) {
          if (source.hasOwnProperty(prop)) {
              if (target[prop] && typeof source[prop] === 'object') {
                  deepObjectExtend(target[prop], source[prop]);
              }
              else {
                  target[prop] = source[prop];
              }
          }
      }
      return target;
  };
  if (Temps && Temps.modFields) {
    const pages = form.get('pages');
    const fieldsToBlock = payload.State.Temps.modFields;

    pages.map((page) => {
      const sections = page.sections;
      sections.map((section) => {
        const elements = section.elements;
        elements.map((element) => {
          fieldsToBlock.filter((field) => {
            if (field.name === element.name) {
              //element = _.extendOwn({}, element, field);
              deepObjectExtend(element, field);
              return false;
            }
            return field;
          });
          return element;
        });
        return section;
      });
      return page;
    });
  }

  if (Temps && Temps.modSections) {
    const pages = form.get('pages');
    const sectionsToBlock = payload.State.Temps.modSections;

    pages.map((page) => {
      const sections = page.sections;
      sections.map((section) => {
        sectionsToBlock.map((sectionBlock) => {
          if (section.name === sectionBlock.name) {
            const elements = section.elements;
            elements.map((element) => {
              delete sectionBlock.name;
              deepObjectExtend(element, sectionBlock);
              //element = _.extendOwn(element, sectionBlock);
            });
          }
          return sectionBlock;
        });
        return section;
      });
    });
  }
};

exports.select = function Select(socket, io, pkg) {
  debug('select:step', pkg.positionId);

  const tenantName = socket.get('tenant');

  let step = new Steps({ db, tenantName: socket.get('tenant'), _id: pkg.positionId });

  step.fetch().then(() => {
    const position = step.get('position');
    const payload = position.payload;

    step = position.activity;

    const formId = step.form.id;
    const bappId = position.bappId;
    const processId = position.processId;


    const form = new Forms({ tenantName: socket.get('tenant'), bapp_id: bappId, form_id: formId, db });
    return form.fetch().then(() => {
      const formName = form.get('name');
      let data = payload.Form[formName] || {};

      if(formName === 'Personal'){
        let code = data.DatosGenerales.Clave;

        if(code && code != ""){
          let url = `/subtypes/search?tenant_id=${socket.get('tenant')}&subType=${formName}&key=${code}`;

          const kernel = require('request-json').createClient(`http://${socket.get('kernel_ip')}/`);

          kernel.get(url, (err, response, body) => {
            if (err) return next(err);
            if (body.status === 200) {
              const item = body.response[0];

              data.ReglaNegocio.SDI = item.properties.SDI;

              // const bapp = new BusinessApps({ db, tenantName: socket.get('tenant') });
              // const catalogName = 'Personal'

              // if(item){
              //   bapp.businessApps.find({ 'forms.name': catalogName }).select('diagrams forms').lean().then((bapps) => {
              //     let foundDiagram;
              //     let finalBapp;

              //     for (let bapp of bapps) {
              //       const diagrams = bapp.diagrams;
              //       for (let diagram of diagrams) {
              //         const pools = diagram.pools;
              //         for (let pool of pools) {
              //           const droppableZones = pool.droppableZones;
              //           for (let zone of droppableZones) {
              //             const elements = zone.elementsCollection;
              //             for (let element of elements) {
              //               if (element.subtype === 'user' && element.form && element.form.name === catalogName) {
              //                 foundDiagram = diagram;
              //                 finalBapp = bapp;
              //               }
              //             }
              //           }
              //         }
              //       }
              //     }

              //     const bappId = finalBapp._id;

              //     const form = _.findWhere(finalBapp.forms, { name: catalogName });
              //         if (!form) return socket.error(new Error('Form Not Found'));

              //     const catData = reverseMapper(item, form);

              //     const actionLeft = [];
              //     let actions = createActions([], [actionLeft]);
              //     const presetActions = customActions({ stepId: pkg.positionId });
              //     actions = createActions([], [actions, presetActions]);

              //     const packet = {
              //       form: form,
              //       data: catData,
              //       actions,
              //     };

              //     socket.emit('form:show', packet);
              //   });
              // }

             }
          });
        }
      }

      mergeFormState(form, payload, formName);
      mergeFormData(data, payload, formName);

      // TODO implement hide sections/fields
      mergeFromTemps(form, payload);

      const actionLeft = [];

      if (payload.State && payload.State.leftActions) {
        for (let action of payload.State.leftActions) {
          actionLeft.push(action);
        }
      }
      // Merge actions
      let actions = createActions([], [actionLeft]);
      customActions(socket, payload, { stepId: pkg.positionId, stepLabel: step.label, formName: formName }, (error, presetActions) => {
        if (error) {
          socket.error(err);
        }
        actions = createActions([], [actions, presetActions]);
        const tenant = db.useDb(socket.get('tenant'));
        const model = tenant.model('data', dataSchema);
        /*
      if(formName != 'Personal'){
        model.findOne({ inboxId: pkg.inboxId }).then((result) => {
          let finalData;

          if (result) finalData = _.extend(data, result.data);
          else finalData = data;

          const packet = {
            form: form.toJSON(),
            data: finalData,
            actions,
          };
          socket.emit('form:show', packet);
        }).catch((err) => {
          socket.error(err);
        });
      }
      else{
        let finalData;

        finalData = data;
        const packet = {
            form: form.toJSON(),
            data: finalData,
            actions,
          };

        socket.emit('form:show', packet);
      }
    });
  })*/
        model.findOne({ inboxId: pkg.inboxId }).then((result) => {
          let finalData;

          if (result) finalData = _.extend(data, result.data);
          else finalData = data;
          const packet = {
            form: form.toJSON(),
            data: finalData,
            actions,
          };
          socket.emit('form:show', packet);
        }).catch((err) => {
          socket.error(err);
        });
      });
    });
  })
.catch((err) => {
    socket.error(err);
  });
};

exports.createAccess = function access(req, res, next) {
  const body = req.body;
  const stepId = req.params._id;
  const bappId = body.bapp_id;
  const diagramId = body.diagram_id;
  const tenant = req.user.tenant;
  const users = body.users;
  const byOmission = body.byOmission || req.user._id;

  saveAccess({ stepId, bappId, diagramId, tenant, users, byOmission })
  .then(() => {
    res.send({
      status: 200,
      message: 'Lista de accesos actualizada con éxito',
    });
  })
  .catch(err => {
    next(err);
  });
};

exports.showAccess = function showAccess(req, res, next) {
  const tenant = req.user.tenant;
  const stepId = req.params._id;
  const bappId = req.params.bapp_id;
  const diagramId = req.params.diagram_id;
  const userId = req.user._id;
  getAccess({ tenant, stepId, bappId, diagramId, userId })
  .then(result => {
    res.send(result);
  }).catch(err => {
    next(err);
  });
};
