const debug = require('debug')('socket');
const session = require('express-session');
const connectRedis = require('connect-redis');
const cookieParser = require('cookie-parser');
const API_AUTH = 'http://auth.g4.intelisis.co/v1';
const request = require('request');

const RedisStore = connectRedis(session);
const store = new RedisStore(__config.session.redis);

const getTenant = (options) => {
  const requestOptions = {
    url: `${API_AUTH}/user/tenants?_id=${options.socket.get('_id')}`,
    json: true,
    headers: {
      cookie: options.cookie,
    },
  };

  return new Promise((resolve, reject) => {
    request(requestOptions, (err, response, body) => {
      if (err) return reject(err);
      return resolve(body);
    });
  });
};

const validateSession = (socket, accept, sessionId) => {
  // check in redis if the sessionId exists
  store.get(sessionId, (err, data) => {
    if (err) {
      debug(err);
      return accept(new Error('InternalServerError'));
    }

    // if the session is valid
    if (data && data.passport && data.passport.user && JSON.stringify(data.passport.user) !== '{}') {
      const user = data.passport.user;
      const email = user.accounts[0].email;
      const userId = user._id;

      socket.set('user', user);
      socket.set('_id', userId);
      socket.set('email', email);

      debug('Authorized.', email);

      const joinUser = (userToJoin) => {
        // if the user has a tenant selected
        socket.set('tenant', userToJoin.tenant.database);
        socket.set('kernel_ip', userToJoin.tenant.kernel_ip || '');
        debug('Tenant', socket.get('tenant'), socket.get('kernel_ip'), email);

        // join user channel
        socket.join(userId, () => {
          accept();
        });
      };
      // check if the user has a tenant selected or not
      if (!user.tenant) {
        // if the user has no tenant selected
        getTenant({ cookie: `session.id=s%3${sessionId}`, socket }).then((result) => {
          if (!result.tenants || result.tenants.length === 0) {
            debug('Invalid Tenant', email);
            accept(new Error('InvalidTenant'));
          }
          user.tenant = result.tenants[0];
          joinUser(user);
        })
        .catch(() => {
          debug('Error to assign tenant automatic');
          accept(new Error('Error to assign tenant automatic'));
        });
      } else {
        joinUser(user);
      }
    } else {
      // if the user is not authenticated
      debug('Unauthorized');
      return accept(new Error('Unauthorized'));
    }
  });
};

module.exports = function () {
  return (socket, accept) => {
    debug('Authorizing ...');

    // mobile validation TODO
    if (socket.handshake.query.mobile) {
      const sessionId = socket.handshake.query.sessionId;
      validateSession(socket, accept, sessionId);
    } else {
      // desktop, browser validation
      // parse request cookies
      cookieParser()(socket.handshake, {}, (cookieError) => {
        if (cookieError) {
          debug(cookieError);
          return accept(cookieError);
        }

        // get the parsed cookies
        const result = socket.handshake.cookies;

        // figure the sessionId
        const sessionString = result['session.id'];
        if (!sessionString) {
          debug('Unauthorized');
          return accept(new Error('Unauthorized'));
        }

        const sessionId = sessionString.split('.')[0].split(':')[1];

        validateSession(socket, accept, sessionId);
      });
    }
  };
};

setTimeout(() => {
  store.client.ping();
  store.client.on('error', (err) => {
    debug(err);
  });
}, 5000);
